﻿using System;
using System.Collections.Generic;
using System.Text;

using Com.Piwigo.Lib.DTO;
using System.Web;
using Com.Piwigo.Lib.Proxy.Response;
using Com.Piwigo.Lib.Proxy.Helper;

namespace Com.Piwigo.Lib.Proxy
{
    /// <summary>
    /// pwg.categories.add              pwg_albums_add
    /// pwg.categories.delete           pwg_albums_delete
    /// pwg.categories.getAdminList     pwg_albums_getAdminList
    /// pwg.categories.getImages        pwg_albums_getImages
    /// pwg.categories.getList          pwg_albums_getList
    /// pwg.categories.move             pwg_albums_move
    /// pwg.categories.setInfo          pwg_albums_setInfo
    /// </summary>
    static internal class PwgAlbumsProxy
    {
        /// <summary>
        /// pwg.Albums.getList
        /// </summary>
        /// <param name="AlbumId"></param>
        /// <param name="Recursive"></param>
        /// <param name="PublicOnly"></param>
        /// <returns></returns>
        static internal PwgAlbumsProxyResponse pwg_albums_getList(Int32? AlbumId, 
            Boolean? Recursive, Boolean? PublicOnly)
        {
            PwgAlbumsProxyResponse response = null;
            try
            {
                StringBuilder data = new StringBuilder();
                Boolean firstOcc = true;

                PwgProxyReponseHelper.buildQueryFromValue<Int32>(AlbumId, "cat_id", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Boolean>(Recursive, "recursive", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Boolean>(PublicOnly, "public", ref firstOcc, data);

                response = PwgGenericProxy<PwgAlbumsProxyResponse>.Get(
                                PwgConfigProxy.PwgServeurUri,
                                "pwg.categories.getList", data.ToString());
            }
            catch (Exception ex)
            {
                throw new PwgProxyException("pwg_albums_getList", ex);
            }
            return response;

        }

        /// <summary>
        /// pwg.Albums.getAdminList
        /// </summary>
        /// <returns></returns>
        static internal PwgAlbumsProxyResponse pwg_albums_getAdminList()
        {
            PwgAlbumsProxyResponse response = null;
            try
            {
                response = PwgGenericProxy<PwgAlbumsProxyResponse>.Get(
                                PwgConfigProxy.PwgServeurUri,
                                "pwg.categories.getAdminList", null);
            }
            catch (Exception ex)
            {
                throw new PwgProxyException("pwg_albums_getAdminList", ex);
            }
            return response;

        }

        /// <summary>
        /// pwg.Albums.add
        /// </summary>
        /// <param name="AlbumName"></param>
        /// <param name="IdUpperCat"></param>
        /// <returns></returns>
        static internal PwgAddRequestProxyResponse pwg_albums_add(String AlbumName, Int32? IdUpperCat)
        {
            PwgAddRequestProxyResponse response = null;
            try
            {
                StringBuilder data = new StringBuilder();
                data.Append("method=pwg.categories.add");
                Boolean firstOcc = false;
                PwgProxyReponseHelper.buildQueryFromValue<String>(AlbumName, "name", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(IdUpperCat, "parent", ref firstOcc, data);

                response = PwgGenericProxy<PwgAddRequestProxyResponse>.Post(
                    PwgConfigProxy.PwgServeurUri,
                    data.ToString());
            }
            catch (Exception ex)
            {
                throw new PwgProxyException("pwg_albums_add", ex);
            }

            return response;
        }

        /// <summary>
        /// pwg.Albums.setInfo
        /// </summary>
        /// <param name="IdCat"></param>
        /// <param name="AlbumName"></param>
        /// <param name="AlbumComment"></param>
        /// <returns></returns>
        static internal PwgBaseProxyReponse pwg_albums_setInfo(Int32 IdCat, String AlbumName, String AlbumComment)
        {
            PwgBaseProxyReponse response = null;
            try
            {
                StringBuilder data = new StringBuilder();
                data.Append("method=pwg.categories.setInfo");
                Boolean firstOcc = false;
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(IdCat, "category_id", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<String>(AlbumName, "name", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<String>(AlbumComment, "comment", ref firstOcc, data);

                response = PwgGenericProxy<PwgAddRequestProxyResponse>.Post(
                    PwgConfigProxy.PwgServeurUri,
                    data.ToString());
            }
            catch (Exception ex)
            {
                throw new PwgProxyException("pwg_albums_setInfo", ex);
            }

            return response;
        }

        /// <summary>
        /// pwg.Albums.getImages
        /// </summary>
        /// <param name="catId"></param>
        /// <param name="recursive"></param>
        /// <param name="perPage"></param>
        /// <param name="Page"></param>
        /// <param name="Order"></param>
        /// <param name="minRate"></param>
        /// <param name="maxRate"></param>
        /// <param name="minHit"></param>
        /// <param name="maxHit"></param>
        /// <param name="minDateAvailable"></param>
        /// <param name="maxDateAvailable"></param>
        /// <param name="minDateCreated"></param>
        /// <param name="maxDateCreated"></param>
        /// <param name="minRatio"></param>
        /// <param name="maxRatio"></param>
        /// <param name="withThumbnail"></param>
        /// <returns></returns>
        static internal PwgImagesProxyResponse pwg_albums_getImages(Int32 catId,
                                                            Boolean? recursive,
                                                            Int32? perPage,
                                                            Int32? Page,
                                                            Int32? Order,
                                                            Int32? minRate,
                                                            Int32? maxRate,
                                                            Int32? minHit,
                                                            Int32? maxHit,
                                                            DateTime? minDateAvailable,
                                                            DateTime? maxDateAvailable,
                                                            DateTime? minDateCreated,
                                                            DateTime? maxDateCreated,
                                                            Int32? minRatio,
                                                            Int32? maxRatio,
                                                            Boolean? withThumbnail)
        {
            PwgImagesProxyResponse response = null;
            try
            {
                StringBuilder data = new StringBuilder();
                Boolean firstOcc = true;
                data.AppendFormat("method={0}", "pwg.categories.getImages");
                firstOcc = false;
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(catId, "cat_id", ref firstOcc, data);

                PwgProxyReponseHelper.buildQueryFromValue<Boolean>(recursive, "recursive", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(perPage, "per_page", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(Page, "page", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(Order, "order", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(minRate, "f_min_rate", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(maxRate, "f_max_rate", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(minHit, "f_min_hit", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(maxHit, "f_max_hit", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<DateTime>(minDateAvailable, "f_min_date_available", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<DateTime>(maxDateAvailable, "f_max_date_available", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<DateTime>(minDateCreated, "f_min_date_created", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<DateTime>(maxDateCreated, "f_max_date_created", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(minRatio, "f_min_ratio", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(maxRatio, "f_max_ratio", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Boolean>(withThumbnail, "f_with_thumbnail", ref firstOcc, data);

                response = PwgGenericProxy<PwgImagesProxyResponse>.Post(
                                PwgConfigProxy.PwgServeurUri,
                                data.ToString());
            }
            catch (Exception ex)
            {
                throw new PwgProxyException("pwg_albums_getImages", ex);
            }
            return response;

        }

        /// <summary>
        /// pwg.Albums.delete
        /// </summary>
        /// <param name="AlbumsId"></param>
        /// <param name="SecurityToken"></param>
        /// <param name="PhotoDeletionMode"></param>
        /// <returns></returns>
        static internal PwgBaseProxyReponse pwg_albums_delete(Int32 AlbumId,
                                                                  String SecurityToken,
                                                                  PwgAlbumPhotoDeletionModeEnum PhotoDeletionMode)
        {
            PwgBaseProxyReponse response = null;
            try
            {
                StringBuilder data = new StringBuilder();
                Boolean firstOcc = true;
                data.Append("method=pwg.categories.delete");
                firstOcc = false;
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(AlbumId, "category_id", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<String>(SecurityToken, "pwg_token", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<PwgAlbumPhotoDeletionModeEnum>(PhotoDeletionMode, "photo_deletion_mode", ref firstOcc, data);

                response = PwgGenericProxy<PwgBaseProxyReponse>.Post(
                                PwgConfigProxy.PwgServeurUri,
                                data.ToString());
            }
            catch (Exception ex)
            {
                throw new PwgProxyException("pwg_albums_delete", ex);
            }

            return response;
        }

        static internal PwgBaseProxyReponse pwg_albums_move(List<Int32> AlbumsId,
                                                                Int32 ParentAlbumId,
                                                                String SecurityToken)
        {
            PwgBaseProxyReponse response = null;
            try
            {
                StringBuilder data = new StringBuilder();
                Boolean firstOcc = true;
                data.Append("method=pwg.categories.move");
                firstOcc = false;
                PwgProxyReponseHelper.buildQueryFromArray<Int32>(AlbumsId, "category_id", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<Int32>(ParentAlbumId, "parent", ref firstOcc, data);
                PwgProxyReponseHelper.buildQueryFromValue<String>(SecurityToken, "pwg_token", ref firstOcc, data);

                response = PwgGenericProxy<PwgBaseProxyReponse>.Post(
                                PwgConfigProxy.PwgServeurUri,
                                data.ToString());
            }
            catch (Exception ex)
            {
                throw new PwgProxyException("pwg_albums_delete", ex);
            }

            return response;
        }
    }

}

