﻿using System;
using System.Collections.Generic;
using System.Text;

using Com.Piwigo.Lib.DTO;
using Com.Piwigo.Lib.Proxy;
using Com.Piwigo.Lib.Proxy.Response;
using Com.Piwigo.Lib.DTO.Helper;
using Com.Piwigo.Lib.IService;

namespace Com.Piwigo.Lib.Service
{

    internal sealed class PwgAlbumsService : IPwgAlbumsService
    {
        /// <summary>
        /// Get the list of Album the user is connected can access
        /// </summary>
        /// <param name="sortedByCounter"></param>
        /// <returns></returns>
        public List<PwgAlbum> GetListOfAlbum(Int32? CatgeroryId, Boolean? Recursive, Boolean? PublicOnly)
        {
            List<PwgAlbum> returnValue = new List<PwgAlbum>();

            try
            {
                PwgAlbumsProxyResponse response = PwgAlbumsProxy.pwg_albums_getList(CatgeroryId, Recursive, PublicOnly);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("GetListOfAlbum, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("GetListOfAlbum : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = ConvertProxyResponseToDTO(response, CatgeroryId);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("GetListOfAlbum : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        /// <summary>
        /// Get List of all Album, must be logged as admin
        /// </summary>
        /// <returns></returns>
        public List<PwgAlbum> GetAdminListOfAlbum()
        {
            List<PwgAlbum> returnValue = new List<PwgAlbum>();

            try
            {
                PwgAlbumsProxyResponse response = PwgAlbumsProxy.pwg_albums_getAdminList();

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("GetAdminListOfAlbum, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("GetAdminListOfAlbum : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = ConvertProxyResponseToDTO(response, null);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("GetAdminListOfAlbum : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        /// <summary>
        /// Add a Album to the gallery
        /// </summary>
        /// <param name="AlbumName"></param>
        /// <param name="upperCatId"></param>
        /// <param name="newId"> Id used by the new Album</param>
        /// <param name="messageInfo">Message returned by the server</param>
        /// <returns>false, if a error occurs</returns>
        public Boolean AddAlbum(String AlbumName, Int32? upperCatId, ref Int32 newId, ref String messageInfo)
        {
            Boolean returnValue = false;

            try
            {
                PwgAddRequestProxyResponse response = PwgAlbumsProxy.pwg_albums_add(AlbumName, upperCatId);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("AddAlbum, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("AddAlbum : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = true;
                    newId = response.NewId;
                    messageInfo = response.Message;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("AddAlbum : a error is raised by proxy.", ex);
            }
            return returnValue;
        }


        /// <summary>
        /// Update Album data
        /// </summary>
        /// <param name="IdCat"></param>
        /// <param name="AlbumName"></param>
        /// <param name="AlbumComment"></param>
        /// <returns></returns>
        public Boolean UpdateAlbumInfo(Int32 IdCat, String AlbumName, String AlbumComment)
        {
            Boolean returnValue = false;

            try
            {
                PwgBaseProxyReponse response = PwgAlbumsProxy.pwg_albums_setInfo(IdCat, AlbumName, AlbumComment);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("UpdateAlbumInfo, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("UpdateAlbumInfo : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = true;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("UpdateAlbumInfo : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        /// <summary>
        /// return the list of PwgImage in the Album
        /// </summary>
        /// <param name="AlbumId"></param>
        /// <param name="recursive"></param>
        /// <param name="perPage"></param>
        /// <param name="Page"></param>
        /// <param name="Order"></param>
        /// <param name="minRate"></param>
        /// <param name="maxRate"></param>
        /// <param name="minHit"></param>
        /// <param name="maxHit"></param>
        /// <param name="minDateAvailable"></param>
        /// <param name="maxDateAvailable"></param>
        /// <param name="minDateCreated"></param>
        /// <param name="maxDateCreated"></param>
        /// <param name="minRatio"></param>
        /// <param name="maxRatio"></param>
        /// <param name="withThumbnail"></param>
        /// <param name="PageReturned"></param>
        /// <param name="PerPageeReturned"></param>
        /// <param name="CountReturned"></param>
        /// <returns></returns>
        public List<PwgImage> GetListOfImagesFormAlbum(Int32 AlbumId,
                                                    Boolean? recursive,
                                                    Int32? perPage,
                                                    Int32? Page,
                                                    Int32? Order,
                                                    Int32? minRate,
                                                    Int32? maxRate,
                                                    Int32? minHit,
                                                    Int32? maxHit,
                                                    DateTime? minDateAvailable,
                                                    DateTime? maxDateAvailable,
                                                    DateTime? minDateCreated,
                                                    DateTime? maxDateCreated,
                                                    Int32? minRatio,
                                                    Int32? maxRatio,
                                                    Boolean? withThumbnail,
                                                    ref Int32 PageReturned,
                                                    ref Int32 PerPageeReturned,
                                                    ref Int32 CountReturned)
        {
            List<PwgImage> returnValue = new List<PwgImage>();

            try
            {
                PwgImagesProxyResponse response = PwgAlbumsProxy.pwg_albums_getImages(AlbumId,
                                                            recursive,
                                                            perPage,
                                                            Page,
                                                            Order,
                                                            minRate,
                                                            maxRate,
                                                            minHit,
                                                            maxHit,
                                                            minDateAvailable,
                                                            maxDateAvailable,
                                                            minDateCreated,
                                                            maxDateCreated,
                                                            minRatio,
                                                            maxRatio,
                                                            withThumbnail);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("GetListOfImagesFormAlbum, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("GetListOfImagesFormAlbum : a error occurs during server process.");
                    }
                }
                else
                {
                    PwgImagesService.ConvertProxyResponseToDTO(response, returnValue,
                                                            ref PageReturned,
                                                            ref PerPageeReturned,
                                                            ref CountReturned);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("GetListOfImagesFormAlbum : a error is raised by proxy.", ex);
            }
            return returnValue;
        }
        /// <summary>
        /// Delete a Album
        /// </summary>
        /// <param name="AlbumId"></param>
        /// <param name="SecurityToken"></param>
        /// <param name="PhotoDeletionMode"></param>
        /// <returns></returns>
        public Boolean DeleteAlbum(Int32 AlbumId, String SecurityToken, PwgAlbumPhotoDeletionModeEnum PhotoDeletionMode)
        {
            Boolean returnValue = false;

            try
            {
                if (PhotoDeletionMode == PwgAlbumPhotoDeletionModeEnum.Unknow)
                {
                    PhotoDeletionMode = PwgAlbumPhotoDeletionModeEnum.OrphansOnly;
                }

                PwgBaseProxyReponse response = PwgAlbumsProxy.pwg_albums_delete(AlbumId, SecurityToken, PhotoDeletionMode);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                   if (response.Erreur != null)
                    {
                        throw new PwgServiceException("DeleteAlbum, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("DeleteAlbum : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = true;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("DeleteAlbum : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        public Boolean MoveAlbum(List<Int32> AlbumsId, Int32 ParentAlbumId, String SecurityToken)
        {
            Boolean returnValue = false;

            try
            {
                PwgBaseProxyReponse response = PwgAlbumsProxy.pwg_albums_move(AlbumsId, ParentAlbumId, SecurityToken);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("DeleteAlbum, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("DeleteAlbum : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = true;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("DeleteAlbum : a error is raised by proxy.", ex);
            }
            return returnValue;
        }
        /// <summary>
        /// convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static List<PwgAlbum> ConvertProxyResponseToDTO(PwgAlbumsProxyResponse response, Int32? CatgeroryId)
        {
            List<PwgAlbum> returnValue = new List<PwgAlbum>();

            if (response.Albums != null)
            {
                foreach (PwgAlbumProxyResponse respCat in response.Albums)
                {
                    if ((!CatgeroryId.HasValue) || (CatgeroryId.Value != respCat.Id))
                    {
                        returnValue.Add(ConvertProxyResponseToDTO(respCat));
                    }
                }
            }

            return returnValue;
        }


        /// <summary>
        /// convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static PwgAlbum ConvertProxyResponseToDTO(PwgAlbumProxyResponse response)
        {
            PwgAlbum returnValue = new PwgAlbum();

            try
            {
                returnValue.Id = response.Id;
                returnValue.Name = response.Name;

                returnValue.DeepImagesCount = response.DeepImagesCount;
                returnValue.ImagesCount = response.ImagesCount;
                returnValue.SubAlbumsCount = response.SubAlbumsCount;

                returnValue.UpperAlbumsId = new List<Int32>();
                if (response.UpperAlbumId != null)
                {
                    foreach (String StringId in response.UpperAlbumId.Split(','))
                    {
                        returnValue.UpperAlbumsId.Add(Int32.Parse(StringId));
                    }
                }

                if (response.LastDate != null)
                {
                    DateTime tryDate;
                    if (DateTime.TryParse(response.LastDate, out tryDate))
                    {
                        returnValue.LastDate = tryDate;
                    }
                }

                if (String.IsNullOrEmpty(response.Url))
                {
                    returnValue.Url = null;
                }
                else
                {
                    returnValue.Url = (new UriBuilder(response.Url)).Uri;
                }
            }
            catch (Exception ex)
            {
                throw new PwgServiceException("ConvertProxyResponseToDTO : a error is raised when converting PwgTagProxyResponse.", ex);
            }

            return returnValue;
        }
    }
}
